#include <PID_v1.h>
#include "thermistor.h"
#include "HardwareSerial.h"

// Analog output pin
#define outputPin 5
#define outputPin2 6

// Motor pin
#define m1 3
#define m2 4


// thermistor analog pin
//#define THERMISTORPIN A0
#define NTC_PIN               A0
THERMISTOR thermistor(NTC_PIN,        // Analog pin
                      10000,          // Nominal resistance at 25 ºC
                      3950,           // thermistor's beta coefficient
                      10000);         // Value of the series resistor

// Global temperature reading
uint16_t temp;

// how many samples to take and average
#define NUMSAMPLES 5
// how long between pid/sampling
#define SAMPLETIME 1000
//Define Variables we'll be connecting to
double Setpoint, currentTemp, Output;
//Specify the links and initial tuning parameters
PID myPID(&currentTemp, &Output, &Setpoint,15,.3,0, DIRECT);

void setup()
{
  Serial.begin(9600);
  analogReference(EXTERNAL);
  pinMode(outputPin, OUTPUT);
  pinMode(outputPin2, OUTPUT);
  pinMode(m1, OUTPUT);
  pinMode(m1, OUTPUT);
  
  //initialize PID setpoint *C
  Setpoint = 190;
  //turn the PID on
  myPID.SetMode(AUTOMATIC);
  myPID.SetSampleTime(SAMPLETIME);
  //pid Autotuner
}
void loop()

{
  digitalWrite(m1,HIGH);
  digitalWrite(m2,LOW);
  
  if (Serial.available() > 0)
  {
    // get incoming byte:
    Setpoint = Serial.parseFloat();
  }
  uint8_t i;
  double average = 0;
  // take N samples in a row, with a slight delay
  for (i = 0; i < NUMSAMPLES; i++)
  {
    temp = thermistor.read();
    average += temp;
    delay(10);
  }
  average /= NUMSAMPLES;
  currentTemp=resistanceToC(inputToResistance(average));
  myPID.Compute();
  analogWrite(outputPin, Output);
  analogWrite(outputPin2, Output);

  Serial.print("Set Point: ");
  Serial.print(Setpoint);
  Serial.println(" *C)");
  Serial.print("Temperature: ");
  Serial.print(currentTemp);
  Serial.println(" *C)");
  Serial.print("PID output ");
  Serial.println(Output);
  delay(SAMPLETIME);
}
double inputToResistance(double input) {
  // funtion to convert the input value to resistance
  // the value of the 'other' resistor
  double SERIESRESISTOR = 10000;
  input = 1023 / input - 1;
  return SERIESRESISTOR / input;
}
double resistanceToC(double resistance) {
  // funtion to convert resistance to c
  // temp/resistance for nominal
  double THERMISTORNOMINAL = 118000;
  double TEMPERATURENOMINAL = 25;
  // beta coefficent
  double BCOEFFICIENT = 3950;
  double steinhart;
  steinhart = resistance / THERMISTORNOMINAL;     // (R/Ro)
  steinhart = log(steinhart);                  // ln(R/Ro)
  steinhart /= BCOEFFICIENT;                   // 1/B * ln(R/Ro)
  steinhart += 1.0 / (TEMPERATURENOMINAL + 273.15); // + (1/To)
  steinhart = 1.0 / steinhart;                 // Invert
  steinhart -= 273.15;   // convert to C
  return steinhart;
}

